"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ActionFactory = void 0;
const web3_js_1 = require("@solana/web3.js");
const shared_1 = require("../shared.js");
const relend_action_1 = require("./relend_action.js");
const solana_action_1 = require("./solana_action.js");
class ActionFactory {
    static async buildTxns(data) {
        const { action, network, connection, amount, symbol, publicKey, supplyToEarnApys, supplyToEarnRewards, lendingMarketAddress, referralAccount, nftVoucherAddress, nftInterestAmount, } = data;
        const lendingMarket = lendingMarketAddress ? new web3_js_1.PublicKey(lendingMarketAddress) : undefined;
        let actionFunction;
        switch (network) {
            case shared_1.BlockchainNetwork.RENEC:
                actionFunction = relend_action_1.RelendAction[`build${action}Txns`];
                break;
            case shared_1.BlockchainNetwork.SOLANA:
                actionFunction = solana_action_1.SolanaAction[`build${action}Txns`];
                break;
            default:
                throw new Error("Unsupported blockchain for transaction");
        }
        return await actionFunction({
            network,
            connection,
            amount,
            symbol,
            publicKey,
            supplyToEarnApys,
            supplyToEarnRewards,
            lendingMarketAddress: lendingMarket,
            hostAddress: referralAccount,
            nftVoucherAddress,
            nftInterestAmount,
        });
    }
}
exports.ActionFactory = ActionFactory;
