import { parseReserve } from "@/state";
import { Obligation } from "@/state/obligation";
import { Connection, PublicKey, Transaction, TransactionInstruction, TransactionSignature } from "@solana/web3.js";
import BigNumber from "bignumber.js";
import BN from "bn.js";
import { BlockchainActionOptions, BlockchainNetwork, MarketConfigType, ReserveConfigType, ReserveDataType, SupplyToEarnApy, SupplyToEarnReward } from "../shared";
type ParsedReserve = NonNullable<ReturnType<typeof parseReserve>>["info"];
export declare abstract class BlockchainAction {
    protected network: BlockchainNetwork;
    protected programId: PublicKey;
    protected connection: Connection;
    protected reserve: ReserveConfigType;
    protected lendingMarket: MarketConfigType;
    protected publicKey: PublicKey;
    protected obligationAddress: PublicKey;
    protected obligationAccountInfo: Obligation | null;
    protected userTokenAccountAddress: PublicKey;
    protected userCollateralAccountAddress: PublicKey;
    protected seed: string;
    protected symbol: string;
    protected positions?: number;
    protected amount: BN;
    protected hostAta?: PublicKey;
    protected setupIxs: Array<TransactionInstruction>;
    protected lendingIxs: Array<TransactionInstruction>;
    protected cleanupIxs: Array<TransactionInstruction>;
    protected preTxnIxs: Array<TransactionInstruction>;
    protected postTxnIxs: Array<TransactionInstruction>;
    protected depositReserves: Array<PublicKey>;
    protected borrowReserves: Array<PublicKey>;
    protected supplyToEarnApys: Array<SupplyToEarnApy>;
    protected supplyToEarnRewards: Array<SupplyToEarnReward>;
    protected nftVoucherAddress?: PublicKey;
    protected nftAtaAddress?: PublicKey;
    protected nftInterestAmount?: BN;
    steInstruction?: TransactionInstruction;
    protected constructor(options: BlockchainActionOptions);
    protected getTransactions(): Promise<{
        preLendingTxn: Transaction | null;
        lendingTxn: Transaction | null;
        postLendingTxn: Transaction | null;
    }>;
    private sendSingleTransaction;
    sendTransactions(sendTransaction: (txn: Transaction, connection: Connection) => Promise<TransactionSignature>, preCallback?: () => void, lendingCallback?: () => void, postCallback?: () => void): Promise<string>;
    protected addDepositReserveLiquidityIx(): void;
    protected addRedeemReserveCollateralIx(): void;
    protected addDepositObligationCollateralIx(): void;
    protected addBorrowIx(): void;
    protected formatReserveData(parsedData: NonNullable<ReturnType<typeof parseReserve>>["info"]): ReserveDataType;
    protected calculateSupplyAPY: (reserve: ParsedReserve) => number;
    protected calculateBorrowAPY: (reserve: ParsedReserve) => number;
    protected calculateSupplyAPR(reserve: ParsedReserve): BigNumber;
    protected calculateUtilizationRatio(reserve: ParsedReserve): BigNumber;
    protected calculateBorrowAPR(reserve: ParsedReserve): BigNumber;
}
export {};
