"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConstantsHelper = void 0;
const classes_1 = require("../classes/index.js");
const constants_renec_1 = require("./constants.renec.js");
const constants_solana_1 = require("./constants.solana.js");
class ConstantsHelper {
    static getConfig(network) {
        return this.isRenecChain(network) ? (0, constants_renec_1.getRelendInfo)() : (0, constants_solana_1.getSolanaInfo)();
    }
    static getProgramId(network) {
        return this.isRenecChain(network) ? (0, classes_1.getRelendProgramId)() : (0, classes_1.getSolanaProgramId)();
    }
    static getPDA(network) {
        return this.isRenecChain(network) ? (0, constants_renec_1.getRenecPDAOracle)() : (0, constants_solana_1.getSolanaPDAOracle)();
    }
    static getPricePair(network) {
        return this.isRenecChain(network) ? (0, constants_renec_1.getRenecPricePair)() : (0, constants_solana_1.getSolanaPricePair)();
    }
    static getUSDPricePair(network) {
        return this.isRenecChain(network) ? (0, constants_renec_1.getRenecReUSDPricePair)() : (0, constants_solana_1.getSolanaUSDTPricePair)();
    }
    static getRPCEndpointUrl(network) {
        return this.isRenecChain(network) ? (0, constants_renec_1.getRenecRpcEndpointUrl)() : (0, constants_solana_1.getSolanaRpcEndpointUrl)();
    }
    static isRenecChain(network) {
        return network === classes_1.BlockchainNetwork.RENEC;
    }
    static isSolanaChain(network) {
        return network === classes_1.BlockchainNetwork.SOLANA;
    }
}
exports.ConstantsHelper = ConstantsHelper;
