import { Connection, PublicKey } from "@solana/web3.js";
import { BlockchainNetwork } from "@/classes";
import { Reserve } from "@/state";
import SwitchboardProgram from "@switchboard-xyz/sbv2-lite";
import * as anchor from "@coral-xyz/anchor";
import { PDAInfo } from "@renec-foundation/oracle-sdk";
export interface PricePair {
    network: string;
    version: string;
    quoteSymbol: string;
    quoteToken: string;
    baseSymbol: string;
    baseToken: string;
    maxPrice: string;
    minPrice: string;
    oracleProduct?: PDAInfo;
    oraclePrice?: PDAInfo;
}
export declare const DEFAULT_PRICES: {
    [key: string]: number;
};
type PriceOutput = {
    [address: string]: {
        spotPrice: number;
        emaPrice: number;
    } | undefined;
};
export declare function fetchPrices(network: BlockchainNetwork, parsedReserves: Array<{
    info: Reserve;
    pubkey: PublicKey;
}>, connection: Connection, debug?: boolean): Promise<PriceOutput>;
export declare function fetchPricesRenec(network: BlockchainNetwork, parsedReserves: Array<{
    info: Reserve;
    pubkey: PublicKey;
}>, connection: Connection, debug?: boolean): Promise<PriceOutput>;
export declare function fetchPricesSolana(network: BlockchainNetwork, parsedReserves: Array<{
    info: Reserve;
    pubkey: PublicKey;
}>, connection: Connection, debug?: boolean): Promise<PriceOutput>;
/**
 * Reimplement the load switchboard program function to use our switchboard oracle program id.
 */
export declare function loadSwitchBoardProgram(connection?: Connection, confirmOptions?: anchor.web3.ConfirmOptions): Promise<SwitchboardProgram>;
export {};
