"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadSwitchBoardProgram = exports.fetchPricesSolana = exports.fetchPricesRenec = exports.fetchPrices = exports.DEFAULT_PRICES = void 0;
const web3_js_1 = require("@solana/web3.js");
const classes_1 = require("../../classes/index.js");
const custom_oracle_1 = require("../../utils/custom-oracle/index.js");
const sbv2_lite_1 = __importStar(require("@switchboard-xyz/sbv2-lite"));
const constants_common_1 = require("../constants.common.js");
const client_1 = require("@pythnetwork/client");
const utils_1 = require("./utils.js");
const anchor = __importStar(require("@coral-xyz/anchor"));
const constants_helper_1 = require("../constants.helper.js");
exports.DEFAULT_PRICES = {
    [constants_common_1.RE_BTC]: 42_068.2,
    [constants_common_1.RE_ETH]: 2_267.37,
    [constants_common_1.RENEC]: 0.30234,
    [constants_common_1.RE_VND]: 0.000040760609273283,
    [constants_common_1.RE_NGN]: 0.0006666,
};
const DEFAULT_PRICE_FOR_ALL_TOKEN = 1;
const SBV2_MAINNET = "SW1TCH7qEPTdLsDHRgPuMQjbQxKdH2aBStViMFnt64f";
async function fetchPrices(network, parsedReserves, connection, debug) {
    if (debug)
        console.log("fetchPrices");
    switch (network) {
        case classes_1.BlockchainNetwork.RENEC:
            return fetchPricesRenec(network, parsedReserves, connection, debug);
        case classes_1.BlockchainNetwork.SOLANA:
            return fetchPricesSolana(network, parsedReserves, connection, debug);
        default:
            throw new Error("Unsupported blockchain for transaction");
    }
}
exports.fetchPrices = fetchPrices;
async function fetchPricesRenec(network, parsedReserves, connection, debug) {
    if (debug)
        console.log("fetchPrices");
    const relendInfo = constants_helper_1.ConstantsHelper.getConfig(network);
    const pda = constants_helper_1.ConstantsHelper.getPDA(network);
    const PRICE_PAIRS = constants_helper_1.ConstantsHelper.getPricePair(network);
    const USD_PRICE_PAIR = constants_helper_1.ConstantsHelper.getUSDPricePair(network);
    const lendingMarket = relendInfo.find((market) => market.isPrimary) ?? relendInfo[0];
    const pricePairs = parsedReserves.reduce((acc, reserve) => {
        const symbol = lendingMarket.reserves.find((r) => r.address === reserve.pubkey.toBase58())?.liquidityToken.symbol;
        if (!symbol) {
            return acc;
        }
        if (symbol === constants_common_1.RE_USD || symbol === constants_common_1.USDT) {
            const priceWithAddress = {
                ...USD_PRICE_PAIR,
                reserveAddress: reserve.pubkey.toBase58(),
            };
            acc.push(priceWithAddress);
            return acc;
        }
        let foundPair;
        if (symbol === constants_common_1.RE_VND || symbol === constants_common_1.RE_NGN || symbol === constants_common_1.VND) {
            // because reVND and reNGN is the opposite (reUSD / reVND(reNGN))
            foundPair = PRICE_PAIRS.find((pair) => pair.quoteSymbol === symbol);
        }
        else {
            foundPair = PRICE_PAIRS.find((pair) => pair.baseSymbol === symbol);
        }
        if (!foundPair) {
            return acc;
        }
        const priceWithAddress = {
            ...foundPair,
            reserveAddress: reserve.pubkey.toBase58(),
        };
        acc.push(priceWithAddress);
        return acc;
    }, []);
    const output = {};
    for (const pair of pricePairs) {
        try {
            if ((pair.baseSymbol === constants_common_1.RE_USD && pair.quoteSymbol === constants_common_1.RE_USD) || // renec
                (pair.baseSymbol === constants_common_1.USDT && pair.quoteSymbol === constants_common_1.USDT) // solana
            ) {
                output[pair.reserveAddress] = {
                    spotPrice: 1,
                    emaPrice: 1,
                };
            }
            else {
                const oracleProduct = pda.product(new web3_js_1.PublicKey(pair.quoteToken), new web3_js_1.PublicKey(pair.baseToken));
                const oraclePrice = pda.price(oracleProduct.key);
                const priceOracle = await (0, custom_oracle_1.parsePriceOracle)(connection, oraclePrice.key);
                const productOracle = await (0, custom_oracle_1.parseProductOracle)(connection, oracleProduct.key);
                const expo = productOracle?.expo ?? 0;
                if (debug) {
                    console.log("pair: ", pair);
                    console.log("expo: ", expo);
                    console.log("oracleProduct key", oracleProduct.key.toBase58());
                    console.log("oraclePrice key", oraclePrice.key.toBase58());
                }
                const defaultSymbol = pair.quoteSymbol === constants_common_1.RE_USD ? pair.baseSymbol : pair.quoteSymbol;
                const defaultPrice = exports.DEFAULT_PRICES[defaultSymbol] ?? DEFAULT_PRICE_FOR_ALL_TOKEN;
                let spotPrice;
                let emaPrice;
                if (pair.quoteSymbol === constants_common_1.RE_VND || pair.quoteSymbol === constants_common_1.RE_NGN || pair.quoteSymbol === constants_common_1.VND) {
                    spotPrice =
                        priceOracle && !priceOracle.price.isZero() && priceOracle.price.toNumber() * Math.pow(10, expo) !== 0
                            ? 1 / (priceOracle.price.toNumber() * Math.pow(10, expo))
                            : defaultPrice;
                    emaPrice =
                        priceOracle && !priceOracle.price.isZero() && priceOracle.price.toNumber() * Math.pow(10, expo) !== 0
                            ? 1 / (priceOracle.price.toNumber() * Math.pow(10, expo))
                            : defaultPrice;
                }
                else {
                    spotPrice =
                        priceOracle && !priceOracle.price.isZero()
                            ? priceOracle.price.toNumber() * Math.pow(10, expo)
                            : defaultPrice;
                    emaPrice =
                        priceOracle && !priceOracle.price.isZero()
                            ? priceOracle.price.toNumber() * Math.pow(10, expo)
                            : defaultPrice;
                }
                output[pair.reserveAddress] = {
                    spotPrice,
                    emaPrice,
                };
            }
        }
        catch (err) {
            console.error("error fetching on pair", pair.baseSymbol, pair.quoteSymbol);
            console.error("error", err);
        }
    }
    if (debug)
        console.log("output", output);
    return output;
}
exports.fetchPricesRenec = fetchPricesRenec;
const DEFAULT_PRICES_SOLANA_DEVNET = {
    Fd7r6Hu7GAkV4Tkhd8Mwd947n2vHoXtr5E6LX33NnAYs: 0.0000390625, // VND: 1 / 25600
};
async function fetchPricesSolana(network, parsedReserves, connection, debug) {
    if (debug)
        console.log("fetchPrices");
    const isSolanaChain = constants_helper_1.ConstantsHelper.isSolanaChain(network);
    const switchboardProgram = await sbv2_lite_1.default.load(connection);
    // const switchboardProgram = await loadSwitchBoardProgram(connection)
    const oracles = parsedReserves
        .map((reserve) => reserve.info.liquidity.pythOracle)
        .concat(parsedReserves.map((reserve) => reserve.info.liquidity.switchboardOracle));
    const priceAccounts = await (0, utils_1.getBatchMultipleAccountsInfo)(oracles, connection);
    const renecResult = await fetchPricesRenec(network, parsedReserves, connection, debug);
    return parsedReserves.reduce((acc, reserve, i) => {
        /**
         * Use default prices for unavailable devnet token
         */
        const devnetDefaultPriceKeys = Object.keys(DEFAULT_PRICES_SOLANA_DEVNET);
        if (devnetDefaultPriceKeys.includes(reserve.pubkey.toBase58()) && isSolanaChain && !constants_common_1.IS_MAINNET) {
            return {
                ...acc,
                [reserve.pubkey.toBase58()]: {
                    spotPrice: DEFAULT_PRICES_SOLANA_DEVNET[reserve.pubkey.toBase58()],
                    emaPrice: DEFAULT_PRICES_SOLANA_DEVNET[reserve.pubkey.toBase58()],
                },
            };
        }
        const pythOracleData = priceAccounts[i];
        const switchboardOracleData = priceAccounts[parsedReserves.length + i];
        let priceData;
        // Use RENEC price if no priceData available
        if (renecResult[reserve.pubkey.toBase58()]) {
            priceData = renecResult[reserve.pubkey.toBase58()];
        }
        if (pythOracleData && !priceData) {
            const { price, previousPrice, emaPrice } = (0, client_1.parsePriceData)(pythOracleData.data);
            if (price || previousPrice) {
                // use latest price if available otherwise fallback to previous
                priceData = {
                    spotPrice: price || previousPrice,
                    emaPrice: emaPrice?.value ?? (price || previousPrice),
                };
            }
        }
        // Only attempt to fetch from switchboard if not already available from pyth
        if (!priceData) {
            const rawSb = switchboardOracleData;
            const switchboardData = switchboardOracleData?.data?.slice(1);
            if (rawSb && switchboardData) {
                const owner = rawSb.owner.toString();
                if (owner === SBV2_MAINNET) {
                    const result = switchboardProgram.decodeLatestAggregatorValue(rawSb);
                    priceData = {
                        spotPrice: result?.toNumber() ?? 0,
                        emaPrice: result?.toNumber() ?? 0,
                    };
                }
            }
        }
        return {
            ...acc,
            [reserve.pubkey.toBase58()]: priceData,
        };
    }, {});
}
exports.fetchPricesSolana = fetchPricesSolana;
/**
 * Reimplement the load switchboard program function to use our switchboard oracle program id.
 */
async function loadSwitchBoardProgram(connection = new anchor.web3.Connection(anchor.web3.clusterApiUrl("devnet")), confirmOptions = sbv2_lite_1.default.defaultConfirmOptions) {
    const provider = new anchor.AnchorProvider(connection, new sbv2_lite_1.AnchorWallet(anchor.web3.Keypair.fromSeed(new Uint8Array(32).fill(1))), confirmOptions);
    const anchorIdl = await anchor.Program.fetchIdl(new web3_js_1.PublicKey(classes_1.SWITCHBOARD_PROGRAM_ID), provider);
    if (!anchorIdl) {
        throw new Error(`failed to read devnet idl for ${new web3_js_1.PublicKey(classes_1.SWITCHBOARD_PROGRAM_ID)}`);
    }
    const program = new anchor.Program(anchorIdl, new web3_js_1.PublicKey(classes_1.SWITCHBOARD_PROGRAM_ID), provider);
    return new sbv2_lite_1.default(program, "devnet");
}
exports.loadSwitchBoardProgram = loadSwitchBoardProgram;
