"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.refreshReserveSolanaInstruction = void 0;
const web3_js_1 = require("@solana/web3.js");
const instruction_1 = require("./instruction.js");
const BufferLayout = require("buffer-layout");
/// Accrue interest and update market price of liquidity on a reserve.
///
/// Accounts expected by this instruction:
///
///   0. `[writable]` Reserve account.
///   1. `[]` Clock sysvar.
///   2. `[optional]` Reserve liquidity oracle account.
///                     Required if the reserve currency is not the lending market quote
///                     currency.
const refreshReserveSolanaInstruction = (reserve, solendProgramAddress, oraclePrice, oracleProduct, switchboardFeedAddress) => {
    const dataLayout = BufferLayout.struct([BufferLayout.u8("instruction")]);
    const data = Buffer.alloc(dataLayout.span);
    dataLayout.encode({ instruction: instruction_1.LendingInstruction.RefreshReserve }, data);
    const keys = [{ pubkey: reserve, isSigner: false, isWritable: true }];
    if (oraclePrice) {
        keys.push({ pubkey: oraclePrice, isSigner: false, isWritable: false });
    }
    if (oracleProduct) {
        keys.push({ pubkey: oracleProduct, isSigner: false, isWritable: false });
    }
    if (switchboardFeedAddress) {
        keys.push({
            pubkey: switchboardFeedAddress,
            isSigner: false,
            isWritable: false,
        });
    }
    return new web3_js_1.TransactionInstruction({
        keys,
        programId: solendProgramAddress,
        data,
    });
};
exports.refreshReserveSolanaInstruction = refreshReserveSolanaInstruction;
