"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.repayObligationLiquidityWithNFTVoucherInstruction = void 0;
const web3_js_1 = require("@solana/web3.js");
const spl_token_1 = require("@solana/spl-token");
const instruction_1 = require("./instruction.js");
const core_1 = require("../core/index.js");
const bn_js_1 = __importDefault(require("bn.js"));
const anchor = __importStar(require("@project-serum/anchor0.24.2"));
const Layout = __importStar(require("../utils/layout.js"));
const BufferLayout = require("buffer-layout");
const REPAY_DEBT_VOUCHER_SEED = "REPAY_DEBT_VOUCHER";
/// Repay borrowed liquidity to a reserve with NFT voucher to discount borrow rate. Requires a refreshed obligation and reserve.
///
/// Accounts expected by this instruction:
///
///   0. `[writable]` Source liquidity token account.
///                     Minted by repay reserve liquidity mint.
///                     $authority can transfer $liquidity_amount.
///   1. `[writable]` Destination repay reserve liquidity supply SPL Token account.
///   2. `[writable]` Repay reserve account - refreshed.
///   3. `[writable]` Obligation account - refreshed.
///   4. `[]` Lending market account.
///   5. `[signer]` User transfer authority ($authority).
///   6. `[]` Clock sysvar.
///   7. `[]` Token program id.
///   8. `[]` RepayDebtVoucher account, PDA generated by NFT Voucher that store details about voucher.
///   9. `[writable]` nft mint, The NFT Voucher address.
///   10. `[writable]` nft token account, The user token account that own nft_mint.
///   11. `[]` Oracle price account.
///   12. `[]` Oracle product account.
const repayObligationLiquidityWithNFTVoucherInstruction = (liquidityAmount, sourceLiquidity, destinationLiquidity, repayReserve, obligation, lendingMarket, transferAuthority, solendProgramAddress, nftMintAddress, nftTokenAccount, oraclePrice, oracleProduct) => {
    const dataLayout = BufferLayout.struct([BufferLayout.u8("instruction"), Layout.uint64("liquidityAmount")]);
    const data = Buffer.alloc(dataLayout.span);
    dataLayout.encode({
        instruction: instruction_1.LendingInstruction.RepayObligationLiquidityWithNFTVoucher,
        liquidityAmount: new bn_js_1.default(liquidityAmount),
    }, data);
    const programIdDebtPubkey = new web3_js_1.PublicKey(core_1.PROGRAM_ID_NFT_VOUCHER_ADDRESS);
    const [repayDebtVoucher] = anchor.web3.PublicKey.findProgramAddressSync([Buffer.from(REPAY_DEBT_VOUCHER_SEED), nftMintAddress.toBuffer()], programIdDebtPubkey);
    const keys = [
        { pubkey: sourceLiquidity, isSigner: false, isWritable: true },
        { pubkey: destinationLiquidity, isSigner: false, isWritable: true },
        { pubkey: repayReserve, isSigner: false, isWritable: true },
        { pubkey: obligation, isSigner: false, isWritable: true },
        { pubkey: lendingMarket, isSigner: false, isWritable: false },
        { pubkey: transferAuthority, isSigner: true, isWritable: false },
        { pubkey: spl_token_1.TOKEN_PROGRAM_ID, isSigner: false, isWritable: false },
        { pubkey: repayDebtVoucher, isSigner: false, isWritable: false },
        { pubkey: nftMintAddress, isSigner: false, isWritable: true },
        { pubkey: nftTokenAccount, isSigner: false, isWritable: true },
        { pubkey: oraclePrice, isSigner: false, isWritable: false },
        { pubkey: oracleProduct, isSigner: false, isWritable: false },
    ];
    return new web3_js_1.TransactionInstruction({
        keys,
        programId: solendProgramAddress,
        data,
    });
};
exports.repayObligationLiquidityWithNFTVoucherInstruction = repayObligationLiquidityWithNFTVoucherInstruction;
