"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fetchTokensInfoSolanaMetaplex = exports.fetchTokensInfoSolana = void 0;
const mpl_token_metadata_1 = require("@metaplex-foundation/mpl-token-metadata");
const umi_bundle_defaults_1 = require("@metaplex-foundation/umi-bundle-defaults");
const spl_token_1 = require("@solana/spl-token");
const web3_js_1 = require("@solana/web3.js");
const child_process_1 = require("child_process");
const custom_utl_sdk_1 = __importDefault(require("../utils/custom-utl-sdk/index.js"));
const token_config_1 = require("./config/token.config.js");
const gen_reserve_cli_1 = __importDefault(require("./gen-reserve.cli.js"));
const reserve_rust2json_1 = require("./utils/reserve.rust2json.js");
const umi_1 = require("@metaplex-foundation/umi");
const spl_token_registry_1 = require("@solana/spl-token-registry");
const axios_1 = __importDefault(require("axios"));
const env = "renec_testnet";
const token = "reVND";
function transformToNewFormat(currentJson, tokenInfo, reserve) {
    return {
        liquidityToken: {
            coingeckoID: tokenInfo.name,
            decimals: currentJson.liquidity.mint_decimals,
            logo: tokenInfo.logoURI,
            mint: currentJson.liquidity.mint_pubkey,
            name: tokenInfo.name,
            symbol: tokenInfo.symbol,
            volume24h: 24000,
        },
        pythOracle: currentJson.liquidity.oracle_pubkey,
        switchboardOracle: currentJson.liquidity.second_oracle_pubkey,
        address: reserve,
        collateralMintAddress: currentJson.collateral.mint_pubkey,
        collateralSupplyAddress: currentJson.collateral.supply_pubkey,
        liquidityAddress: currentJson.liquidity.supply_pubkey,
        liquidityFeeReceiverAddress: currentJson.config.fee_receiver,
        userBorrowCap: currentJson.config.borrow_limit,
        userSupplyCap: currentJson.config.deposit_limit,
    };
}
async function getTokenInfo(env, tokenSymbol, liquidityPubkey) {
    switch (env) {
        case "solana_devnet":
        case "solana_phuc_devnet":
        case "solend":
        case "solana_mainnet":
            return (await (0, exports.fetchTokensInfoSolana)(env, tokenSymbol, liquidityPubkey))[liquidityPubkey];
        default:
            return getTokenInfoRenec(liquidityPubkey);
    }
}
async function getTokenInfoRenec(liquidityPubkey) {
    let tokenInfo;
    if (liquidityPubkey === "USDbgSB1DPBCEDt15ppgTZZYiQokMHcvkX6JQRfVNJY" || liquidityPubkey === "4Q89182juiadeFgGw3fupnrwnnDmBhf7e7fHWxnUP3S3") {
        tokenInfo = {
            name: "USDT (reUSD)",
            symbol: "reUSD",
            address: "USDbgSB1DPBCEDt15ppgTZZYiQokMHcvkX6JQRfVNJY",
            decimals: 9,
            logoURI: "https://raw.githubusercontent.com/renec-chain/rpl-token-registry/master/assets/mainnet/4Q89182juiadeFgGw3fupnrwnnDmBhf7e7fHWxnUP3S3/logo.png",
        };
    }
    else if (liquidityPubkey === spl_token_1.NATIVE_MINT.toBase58()) {
        tokenInfo = {
            name: "RENEC",
            symbol: "RENEC",
            address: spl_token_1.NATIVE_MINT.toBase58(),
            decimals: 9,
            logoURI: "https://easytokenbucket.s3.amazonaws.com/nsbr7a24f6ox3xh40obw8jaaxjzt",
        };
    }
    else if (liquidityPubkey === "8NoZYiawFCNhGUSDoLAy3xr2DMCKnnn96vbTvjw8Ykzs") {
        // reVND
        tokenInfo = {
            name: "reVND",
            symbol: "reVND",
            address: "8NoZYiawFCNhGUSDoLAy3xr2DMCKnnn96vbTvjw8Ykzs",
            decimals: 0,
            logoURI: "https://easytokenbucket.s3.amazonaws.com/y5ugt406h4x6nrat5u4s1cbyyaw9",
        };
    }
    else {
        const utl = new custom_utl_sdk_1.default();
        const tokens = await utl.fetchMints("https://raw.githubusercontent.com/renec-chain/rpl-token-registry/master/src/tokens/renec.tokenlist.json", [new web3_js_1.PublicKey(liquidityPubkey)]);
        tokenInfo = tokens[0];
    }
    return tokenInfo;
}
const fetchTokensInfoSolana = async (env, tokenSymbol, liquidityPubkey) => {
    const provider = await new spl_token_registry_1.TokenListProvider().resolve();
    const tokenList = provider.filterByChainId(spl_token_registry_1.ENV.MainnetBeta).getList();
    const tokens = tokenList.filter((token) => tokenSymbol === token.symbol);
    console.log("🚀 ~ tokenSymbol:", tokenSymbol);
    console.log("🚀 ~ tokens:", tokens.length);
    if (tokens.length === 0) {
        return await (0, exports.fetchTokensInfoSolanaMetaplex)(env, liquidityPubkey);
    }
    return Object.fromEntries(tokens
        .map((token) => token
        ? [
            liquidityPubkey,
            {
                name: token.name ?? "",
                symbol: token.symbol,
                logoURI: token.logoURI,
                decimals: token.decimals ?? 0,
            },
        ]
        : [])
        .filter((x) => x.length));
};
exports.fetchTokensInfoSolana = fetchTokensInfoSolana;
const fetchTokensInfoSolanaMetaplex = async (env, liquidityPubkey) => {
    const umi = (0, umi_bundle_defaults_1.createUmi)(token_config_1.envConfigs[env].url).use((0, mpl_token_metadata_1.mplTokenMetadata)());
    const publickey = (0, umi_1.publicKey)(liquidityPubkey);
    const test = await (0, mpl_token_metadata_1.fetchAllDigitalAsset)(umi, [publickey]);
    console.log("🚀 ~ fetchTokensInfoSolanaMetaplex ~ test:", test);
    const asset = test[0];
    console.log("🚀 ~ fetchTokensInfoSolanaMetaplex ~ asset:", asset);
    const fetchAssetMetadata = await axios_1.default.get(asset.metadata.uri);
    return {
        [liquidityPubkey]: {
            name: asset.metadata.name,
            symbol: asset.metadata.symbol,
            logoURI: fetchAssetMetadata.data.image,
            decimals: asset.mint.decimals,
        },
    };
};
exports.fetchTokensInfoSolanaMetaplex = fetchTokensInfoSolanaMetaplex;
function formatNumber(num) {
    if (num >= 1e12) {
        return (num / 1e12).toFixed(1) + "T";
    }
    else if (num >= 1e9) {
        return (num / 1e9).toFixed(1) + "B";
    }
    else if (num >= 1e6) {
        return (num / 1e6).toFixed(1) + "M";
    }
    else if (num >= 1e3) {
        return (num / 1e3).toFixed(1) + "K";
    }
    else {
        return num.toString();
    }
}
;
(async () => {
    const env = "solana_mainnet";
    const tokenSymbols = Object.keys(token_config_1.tokenConfigs[env]);
    console.log("🚀 ~ ; ~ tokenSymbols:", tokenSymbols);
    const outputs = [];
    let counter = 0;
    const totalSymbols = tokenSymbols.length;
    tokenSymbols.forEach((tokenSymbol) => {
        const token = tokenSymbol;
        const { reserve, command } = (0, gen_reserve_cli_1.default)(env)(token);
        console.log("### reserve: ", reserve);
        console.log("### command: ", command);
        (0, child_process_1.exec)(command, async (error, stdout) => {
            if (error) {
                console.error(`Error executing CLI command: ${error}`);
                return;
            }
            const reserveRaw = (0, reserve_rust2json_1.reserveRust2json)(stdout);
            const tokenInfo = await getTokenInfo(env, token, reserveRaw.liquidity.mint_pubkey);
            console.log("🚀 ~ exec ~ token:", token);
            console.log("🚀 ~ exec ~ tokenInfo:", tokenInfo);
            const transformedJSON = transformToNewFormat(reserveRaw, tokenInfo, reserve);
            outputs.push(transformedJSON);
            counter++;
        });
    });
    while (counter < totalSymbols) {
        await new Promise((resolve) => setTimeout(resolve, 300));
    }
    console.log("outputs", outputs);
    /**
     * Log pyth, switchboard oracles
     */
    const oracles = outputs.map((o) => {
        return {
            symbol: o.liquidityToken.symbol,
            pythOracle: o.pythOracle,
            switchboardOracle: o.switchboardOracle,
        };
    });
    // console.log({ oracles })
    /**
     * Log limits
     */
    const limits = outputs.map((o) => {
        return {
            symbol: o.liquidityToken.symbol,
            userBorrowCap: o.userBorrowCap,
            userBorrowCapFormatted: formatNumber(o.userBorrowCap),
            userSupplyCap: o.userSupplyCap,
            userSupplyCapFormatted: formatNumber(o.userSupplyCap),
        };
    });
    // console.log({ limits })
})();
